from __future__ import annotations

import dataclasses
import typing

import ldaptool.decode.arguments
from ldaptool._utils import argclasses


@dataclasses.dataclass(slots=True, kw_only=True)
class Arguments(ldaptool.decode.arguments.Arguments):
    filter: typing.Optional[str] = dataclasses.field(default=None, metadata=argclasses.arg(help="LDAP query filter"))
    find: typing.Optional[str] = dataclasses.field(
        default=None,
        metadata=argclasses.arg(help="Account/Name/Email to search for (builds filter around it)"),
    )
    # TODO: not calling ldapsearch anymore...
    # debug: bool = dataclasses.field(
    #     default=False,
    #     metadata=argclasses.arg("-d", help="Show arguments to ldapsearch"),
    # )
    gc: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(help="Query global catalogue (and forest root as search base)"),
    )
    realm: str = dataclasses.field(metadata=argclasses.arg(required=True, help="Realm to search in"))
    server: typing.Optional[str] = dataclasses.field(
        default=None,
        metadata=argclasses.arg(
            help="""
                Server of realm to connect to
                (attributes like lastLogon are not replicated and can vary between servers)
            """,
        ),
    )
    all: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(
            help="Get all results (pagination) instead of only first 1000",
        ),
    )
    krb: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(
            help="Use kerberos authentication (ticket must be already present)",
        ),
    )
    base: typing.Optional[str] = dataclasses.field(
        default=None,
        metadata=argclasses.arg(
            "-b",
            help="Explicit search base (defaults to root of domain / forest with --gc)",
        ),
    )

    def __post_init__(self) -> None:
        super(Arguments, self).__post_init__()  # super() not working here, unclear why.

        if not self.filter is None:
            if not self.find is None:
                raise SystemExit("Can't use both --find and --filter")
        elif not self.find is None:
            find = self.find
            self.filter = (
                f"(|(sAMAccountName={find})(email={find})(mail={find})(proxyAddresses=smtp:{find})(description={find}))"
            )
        else:
            # probably doesn't like empty filter?
            self.filter = "(objectClass=*)"
